@@ -25,6 +25,9 @@
 #include <unistd.h>
 #import "UnicodeWrapper.h"
 
+// number of tracks in tracklist downloaded in between progress bar updates (MTP only)
+#define TRACKLIST_PROGRESS_UPDATE_INTERVAL	50
+
 // declared so can be accessed from C function progress
 // set to be the outlet statusDisplayer in awakeFromNib
 StatusDisplayer *statusDisplayerGlobal;
@@ -244,6 +247,9 @@ StatusDisplayer *statusDisplayerGlobal;
 				storageID = storageIDs.Storage[0];
 			free(storageIDs.Storage);
 		}
+		
+		// check no handles array
+		params.handles.Handler = NULL;
 	}
 	
 	[self updateDiskSpace];
@@ -329,6 +335,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		{
 			close_device(&ptp_usb, &params, interfaceNumber);
 			ptp_free_deviceinfo(&params.deviceinfo);
+			if (params.handles.Handler != NULL)
+				free(params.handles.Handler);
 		}
 		connected = NO;
 		[self setStatus:STATUS_DISCONNECTED];
@@ -500,26 +508,27 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{		
+		NSDate *date = [NSDate date];
+		
 		PTPObjectInfo oi;
 		unsigned short *unicodevalue = NULL;
 		uint16_t *uint16value = NULL;
 		uint32_t *uint32value = NULL;
 		int ret;
 		
-		if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+		if (params.handles.Handler == NULL)
 		{
-			// todo: print error
-			NSLog(@"Could not get object handles...");
-			return nil;
+			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+			{
+				// todo: print error
+				NSLog(@"Could not get object handles...");
+				return nil;
+			}
 		}
 		int i = 0;
 		for (i = 0; i < params.handles.n; i++) {
 			if (ptp_getobjectinfo(&params, params.handles.Handler[i], &oi) == PTP_RC_OK)
 			{
-				/*
-				if (oi.Filename != NULL)
-					NSLog(@"handle: 0x%08.x, filename: %s", params.handles.Handler[i], oi.Filename); */
-				
 				if (oi.ObjectFormat == PTP_OFC_Association || 
 						(oi.ObjectFormat != PTP_OFC_WAV && oi.ObjectFormat != PTP_OFC_MP3 && oi.ObjectFormat != PTP_OFC_WMA))
 					continue;
@@ -611,8 +620,22 @@ StatusDisplayer *statusDisplayerGlobal;
 				
 				[cachedTrackList addObject:track];
 				[track release];
+				
+				// update progress bar
+				
+				if (i % TRACKLIST_PROGRESS_UPDATE_INTERVAL == 0)
+				{
+					double progressPercent = (double)i*(double)100.0 / (double)params.handles.n;
+					
+					[statusDisplayerGlobal updateTaskProgress:progressPercent];
+				}
 			}
 		}
+		[statusDisplayerGlobal updateTaskProgress:100.0];
+		
+		double time = -[date timeIntervalSinceNow];
+		
+		NSLog(@"getting tracklist took %f seconds", time);
 	}
 	
 	downloadingTracks = NO;
@@ -670,7 +693,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		}
 		
 		if (send_file(&params, [[track fullPath] fileSystemRepresentation], [[[track fullPath] lastPathComponent] fileSystemRepresentation],
-									format, progressPTP, &handle) != PTP_RC_OK)
+									format, &progressPTP, &handle) != PTP_RC_OK)
 		{
 			NSString *error = @"Error uploading file";
 			NSLog(error);
@@ -969,11 +992,14 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		Directory *baseDir = [[Directory alloc] init];
 		
-		if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+		if (params.handles.Handler == NULL)
 		{
-			// todo: print error
-			NSLog(@"Could not get object handles...");
-			return nil;
+			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+			{
+				// todo: print error
+				NSLog(@"Could not get object handles...");
+				return nil;
+			}
 		}
 		int i = 0;
 		for (i = 0; i < params.handles.n; i++) {
@@ -1093,7 +1119,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		uint32_t handle = 0;
 		uint16_t ret = send_file(&params, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] fileSystemRepresentation],
-														 objectFormat, progressPTP, &handle);
+														 objectFormat, &progressPTP, &handle);
 		if (ret != PTP_RC_OK)
 		{
 			/* we might get an (MTP error) object too large here */
@@ -1303,12 +1329,15 @@ StatusDisplayer *statusDisplayerGlobal;
 	{
 		PTPObjectInfo oi;
 		
-		// todo: can we just ask for PTP_OFC_AbstractAudioVideoPlaylist type?
-		if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+		// we can't get by type because unsupported on Creative devices (we only want PTP_OFC_AbstractAudioVideoPlaylist)
+		if (params.handles.Handler == NULL)
 		{
-			// todo: print error
-			NSLog(@"Could not get playlist object handles...");
-			return nil;
+			if (ptp_getobjecthandles(&params,PTP_GOH_ALL_STORAGE, PTP_GOH_ALL_FORMATS, PTP_GOH_ALL_ASSOCS, &params.handles) != PTP_RC_OK)
+			{
+				// todo: print error
+				NSLog(@"Could not get playlist object handles...");
+				return nil;
+			}
 		}
 		int i = 0;
 		for (i = 0; i < params.handles.n; i++) {
