@@ -8,6 +8,10 @@
 
 #import "WMATagger.h"
 #include "wmaread.h"
+// for stat
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 @implementation WMATagger
 
@@ -19,19 +23,19 @@
 
 + (Track *)readTrack:(NSString *)filename
 {
+	struct stat sb;
 	// check file exists
-	NSFileManager *fileManager = [[NSFileManager alloc] init];
-	// this is used below to get filesize
-	NSDictionary *fileAttributes = [fileManager fileAttributesAtPath:filename traverseLink:YES];
-	if (fileAttributes == nil)
+	if (stat([filename fileSystemRepresentation], &sb) == -1)
 		return nil;
 	
-	Track *track = [[Track alloc] init];
+	// N.B. cannot use NSFileManager as this is not thread safe
 		
-	[track setFilesize:[[fileAttributes objectForKey:NSFileSize] intValue]];
+	Track *track = [[Track alloc] init];
+	
+	[track setFilesize:sb.st_size];
 	[track setCodec:CODEC_WMA];
 	[track setFullPath:filename];
-
+	
 	metadata_t meta;
 	// make sure all pointers are NULL
 	memset(&meta, 0, sizeof(metadata_t));
