@@ -104,6 +104,9 @@
     [self releaseAttributes];
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 	
+		if (path != nil)
+			[path release];
+		
     path = [Path copy];
     int position = 0;
 	parse = 0;
@@ -170,6 +173,8 @@
 
 -(id)initWithPath:(NSString *)Path genreDictionary:(NSMutableDictionary *)Dictionary
 {
+	path = nil;
+	
     if (![self initWithGenreList:Dictionary]) return NULL;
     [self examineFile:Path];
     return self;
@@ -871,7 +876,7 @@
 }
 
 - (void)dealloc
-{
+{	
     [self releaseAttributes];
     if (externalDictionary != YES) 
         if (dataDictionary != NULL) [dataDictionary release];
