@@ -56,27 +56,31 @@
 
 - (int)readPackedLengthFrom:(int)Offset
 {
-    int val = 0;
-    int i;
-    const int MAXVAL = 268435456; //2^28
-    // For each byte of the first 4 bytes in the string...
-    
-    for (i = 0; i < 4; ++i)
-    {// ...append the last 7 bits to the end of the temp integer...
-        val = val * 128;
-        val += Buffer[currentFramePosition + i + Offset];
-    }
-    if (val > MAXVAL) val = MAXVAL;
+	int val = 0;
+	int i;
+	const int MAXVAL = 0x7fffffff; //2^31 - we have signed ints here but prob should be unsigned
+																 // don't worry though as this is against the spec anyway (spec says
+																 // should be 2^28
+	
+																// For each byte of the first 4 bytes in the string...
+	
+	for (i = 0; i < 4; ++i)
+	{// ...append the last 7 bits to the end of the temp integer...
+		/// NB this SHOULD  be 128 according to the spec, but noone seems to do that! Changed by RAL 20060110
+		val = val * 256;
+		val += Buffer[currentFramePosition + i + Offset];
+	}
+	if (val > MAXVAL) val = MAXVAL;
 	if (val > tagLength - frameOffset - currentFramePosition) {
 		NSLog(@"Problem encountered parsing v2 frame:  frame length value longer than tag length, guessing correct frame length");
 		val = tagLength - frameOffset - currentFramePosition;
 	}
 	if (val > MAXUNCOMPRESSEDFRAMESIZE) {
-			val = MAXUNCOMPRESSEDFRAMESIZE;
-			NSLog(@"Warning frame size > maximum allowable frame size, clipping frame to %i bytes.",MAXUNCOMPRESSEDFRAMESIZE);
+		val = MAXUNCOMPRESSEDFRAMESIZE;
+		NSLog(@"Warning frame size > maximum allowable frame size, clipping frame to %i bytes.",MAXUNCOMPRESSEDFRAMESIZE);
 	}
-
-    return val;
+	
+	return val;
 }
 
 -(BOOL)nextFrame:(BOOL)atStart
