@@ -31,6 +31,16 @@
 
 
 @implementation id3V2Tag
+
+-(id)init
+{
+	if (self = [super init])
+	{
+		path = NULL;
+	}
+	return self;
+}
+
 -(id)initWithFrameDictionary:(NSDictionary *)Dictionary
 {
     if (!(self = [super init])) return self;
@@ -72,9 +82,7 @@
 
 -(BOOL)openPath:(NSString *)Path
 {
-    id old = path;
     path = [Path copy];
-    if (old != NULL) [old release];
     //ID3 tag header variables
     present = NO;
     majorVersion = 0;
@@ -177,11 +185,11 @@
             } else {
                 offset = start + 10 + extendedHeaderLength;
             }
-            if ([self tagUnsynch]) 
+            /*if ([self tagUnsynch]) 
             {
                 v2Tag = [self desynchData:v2Tag offset:offset];
                 offset = 0;
-            }
+            }*/
             // create a processing object for the tag version
            
             switch (majorVersion)
@@ -380,7 +388,6 @@
     }
     [file closeFile];
     [sourceFile closeFile];
-    [file release];
     [fileManager removeFileAtPath:path handler:NULL];
     [fileManager movePath:writePath toPath:path handler:NULL];
     [fileManager removeFileAtPath:writePath handler:NULL];
@@ -403,7 +410,8 @@
 }
 
 -(BOOL)setPath:(NSString *)Path {
-	[path release];
+	if (path != NULL)
+		[path release];
 	path = [Path copy];
 	return YES;
 }
@@ -1179,7 +1187,7 @@
                 NSString * yearString = [anObject getTextFromFrame];
                 NSArray *listItems = [yearString componentsSeparatedByString:@"-"];
                 yearString = [listItems objectAtIndex:0];
-								// BUG: listItems must not be released here. Fixed by RAL 2006-01-07
+								// BUG: listItems must not be released. Fixed by RAL 2006-01-07
                 ///////[listItems release];
                 return [yearString intValue];
             }
