@@ -485,7 +485,7 @@ ptp_unpack_DPV (PTPParams *params, char* data, void** value, uint16_t datatype)
 			break;
 		}
 		default:
-			printf("data type 0x%.04x not supported by ptp_unpack_DPV\n", datatype);
+			//printf("data type 0x%.04x not supported by ptp_unpack_DPV\n", datatype);
 			return 0;
 	}
 }
